import json
import logging
import os
import traceback
from datetime import datetime

from config import LOGGING_LEVEL, ENV


class CustomFormatter(logging.Formatter):
    def format(self, record: logging.LogRecord) -> str:
        source = ":".join([record.filename, str(record.lineno)])
        if isinstance(record.msg, dict):
            if "_source" in record.msg:  # override call stack
                source = record.msg["_source"]
                message = {k: v for k, v in record.msg.items() if k not in ["_source"]}
            else:
                message = record.msg
        else:
            message = record.getMessage()
        log = {
            'name': record.name,
            'level': record.levelname,
            'source': source,
            'create_time': datetime.fromtimestamp(record.created).strftime('%Y-%m-%d %H:%M:%S.%f'),
            'message': message
        }
        if record.exc_info:
            log["traceback"] = self.formatException(record.exc_info)
        str_log = json.dumps(log, ensure_ascii=False, separators=(',', ':'))
        if (length := len(str_log)) > 131072:
            log = {
                "error": "logging entity too long",
                "length": length,
                "traceback": ''.join(traceback.format_list(traceback.extract_stack()))
            }
            str_log = json.dumps(log)
        if ENV.is_local() and "traceback" in log:  # 方便本地 DEBUG
            str_log += "\n" + log["traceback"]
        return str_log


class CustomStreamHandler(logging.StreamHandler):
    def __init__(self):
        super().__init__()
        self.setFormatter(CustomFormatter())



# LOG_DIR = 'logs'
LOG_DIR = f"{os.path.dirname(os.path.abspath(__file__))}/../../logs/"

if not os.path.exists(LOG_DIR):
    os.mkdir(LOG_DIR)

# paddle 2.5.2

# paddle/distributed/utils/log_utils.py:32 会给 root logger 加一个 StreamHandler
# 本文件会晚于 LAC 初始化，所以在这里将 paddle 设定的 root handler 替换掉
logging.getLogger("root").handlers = [CustomStreamHandler()]

logger: logging.Logger = logging.getLogger('app')

# paddle/distributed/auto_parallel/static/cluster.py:832 会设置 root 的 level
# 将自己的 level 独立设置为 LOGGING_LEVEL
logger.setLevel(LOGGING_LEVEL)


def get_logger(name: str = None):
    if name:
        return logger.getChild(name)
    return logger
